<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Christian Lerrahn <christian.lerrahn@cerebrum.com.au>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 * Hint: use extdeveval to insert/update function index above.
 */

/**
 * SugarCRM Integration SOAP for the 'eu_sugarcrm' extension.
 *
 * @author    Christian Lerrahn <christian.lerrahn@cerebrum.com.au>
 * @package    TYPO3
 * @subpackage    tx_eusugarcrm
 */
class tx_eusugarcrm_SOAP {
  
  // class variables
  public $prefixId = 'tx_eusugarcrm_SOAP';		// Same as class name
  public $scriptRelPath = 'class.tx_eusugarcrm_SOAP.php';	// Path to this script relative to the extension dir.
  public $extKey = 'eu_sugarcrm';	// The extension key.

  protected $soapClient;
  protected $nonWsdlSoapClient;
  protected $sessionID;
  protected $nonWsdlSessionID;
  protected $extConf;

  private $userID;
  private $userPWHash;
    
    /**
     * The constructor of the class
     * Creates the SOAP client object
     *
     * @return   void
     */
    function __construct() {

      global $TYPO3_CONF_VARS;

      // read configuration of extension
      $this->extConf = $TYPO3_CONF_VARS['SVCONF']['sugarCRM']['tx_eusugarcrm_sv1'];

      // prepare connection details for SoapClient
      $soapOptions = array();
      $soapOptions['location'] = $this->extConf['URL'];
      $soapOptions['uri'] = 'http://www.typo3.org/sugarcrm';

      if (empty($soapOptions['location'])) {
	t3lib_div::devLog('No URL configured for SugarCRM SOAP interface.',$extKey);
	return false;
      }
		
      // check if proxy is to be used
      if ($this->extConf['useProxy']) {
	
	// read global proxy settings of TYPO3 install
	$proxyServer = $GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyServer'];
	$proxyAuth = $GLOBALS['TYPO3_CONF_VARS']['SYS']['curlProxyUserPass'];

	// if proxy server set, split it up into its elements
	if (!empty($proxyServer)) {
	  list($pProt,$pHostPort) = explode('//',$proxyServer,2);
	  list($pHost,$pPort) = explode(':',$pHost,2);
	  
	  // store proxy details in SoapClient options
	  $soapOptions['proxy_host'] = $pHost;
	  $soapOptions['proxy_port'] = $pPort;

	  if (!empty($proxyAuth)) {
	    list($pUser,$pPass) = explode(':',$proxyAuth,2);
	    $soapOptions['proxy_login'] = $pUser;
	    $soapOptions['proxy_password'] = $pPass;
	  }
	}
	else {
	  t3lib_div::devLog('Configured to use proxy but no proxy configured in TYPO3 core.',$extKey);
	}
      }
      $this->soapClient = new SoapClient($soapOptions['location'].'?wsdl',$soapOptions);

      // no good communication between PHP5 SOAP and SugarCRM => need a non-WSDL connection for writing :(
      $this->nonWsdlSoapClient = new SoapClient(NULL,$soapOptions);

      // save SugarCRM login credentials in separate class variables
      $this->userID = $this->extConf['userID'];
      $this->userPWHash = md5($this->extConf['userPW']);

    }

    /***************************************
     *                                     *
     * Functions that do NOT require login *
     *                                     *
     ***************************************/

    /**
     * Retrieves server version
     *
     * @return   object   Response from server
     */
    function get_server_version() {

      return $this->soapClient->get_server_version();
            
    }

    /**
     * Retrieves SugarCRM flavour
     *
     * @return   object   Response from server
     */
    function get_sugar_flavor() {
      
      return $this->soapClient->get_sugar_flavor();

    }

    /**
     * Server ping test
     *
     * @param    string   Payload to send in ping (will be echoed back)
     *
     * @return   object   Response from server
     */
    function test($payload) {

      return $this->soapClient->test($payload);
            
    }

    /**
     * Logs into SugarCRM SOAP interface
     *
     * @param    boolean   if true, use WSDL mode, non-WSDL mode otherwise
     * 
     * @return   boolean   true if logged in successfully, false otherwise
     */
    function login($wsdl=true) {

      if ($wsdl) { // WSDL mode (reading)
	$response = $this->soapClient->login(array('user_name' => $this->userID,
						   'password' => $this->userPWHash,
						   'version' => '.01',
						   'application_name' => 'TYPO3: '.$extKey));
      }
      else {
	$response = $this->nonWsdlSoapClient->login(array('user_name' => $this->userID,
							  'password' => $this->userPWHash,
							  'version' => '.01',
							  'application_name' => 'TYPO3: '.$extKey));
      }

      if ($response->id) {
	if ($wsdl) {
	  $this->sessionID = $response->id;
	}
	else {
	  $this->nonWsdlSessionID = $response->id;
	}
	return true;
      }
      else {
	return false;
      }

    }


    /***********************************
     *                                 *
     * Functions that do require login *
     *                                 *
     ***********************************/

    /**
     * Logs out from SugarCRM SOAP interface
     * 
     * @param    boolean   if true, use object in WSDL mode, non-WSDL mode object otherwise
     * 
     * @return   void
     */
    function logout($wsdl=true) {

      if ($wsdl) {
	$response = $this->soapClient->logout($this->sessionID);
	$this->sessionID = NULL;
      }
      else {
	$response = $this->nonWsdlSoapClient->logout($this->nonWsdlSessionID);
	$this->nonWsdlSessionID = NULL;
      }

    }

    /**
     * Retrieve user list from SugarCRM
     * 
     * 
     * @return   object   Server response
     */
    function get_user_list() {

      $response = $this->soapClient->user_list($this->userID,$this->userPWHash);

      return $response;

    }
    
    /**
     * Retrieve module list from SugarCRM
     * 
     * 
     * @return   object   Server response
     */
    function get_module_list() {

      $response = $this->soapClient->get_available_modules($this->sessionID);

      return $response;

    }

    /**
     * Retrieve module details from SugarCRM
     * 
     * @param    string   Module name
     * 
     * @return   object   Server response
     */
    function get_module_fields($mod) {

      $response = $this->soapClient->get_module_fields($this->sessionID,$mod);

      return $response;

    }    

	
    /*********************
     * Reading functions *
     *********************/

    /**
     * Fetches a record based on its ID
     *
     * @param    string  $module: Name of module to fetch record from
     * @param    string  $id: ID of record to fetch
     * @param    array   $selectFields: Fields to be fetched  
     *
     * @return   object  Server response containing list of matching records
     */
    function get_entry($module,$id,$selectFields=array()) {

      $response = $this->soapClient->get_entry($this->sessionID,$module,$id,$selectFields);
      return $response;

    }

    /**
     * Fetches records based on a query
     *
     * @param    string  $module: Name of module to fetch record from
     * @param    string  $query: Query to match
     * @param    string  $orderby: Order matches by field
     * @param    array   $selectFields: Fields to be fetched
     * @param    integer $offset: Offset to apply to search (Skip n matches)
     * @param    integer $max_results: Maximum number of matches
     * @param    integer $deleted: Include deleted records (?)
     *
     * @return   object  Server response containing list of matching records
     */
    function get_entry_list($module,$query,$selectFields=array(),$orderby='',$offset=0,$max_results=0,$deleted=0) {

      $response = $this->soapClient->get_entry_list($this->sessionID,$module,$query,$orderby,$offset,$selectFields,$max_results,$deleted);
      return $response;

    }

    /**
     * Searches for a string in modules
     *
     * @param    string         $searchString: Search string
     * @param    string/array   $modules: List of modules to search in
     * @param    integer        $max: Maximum number or results
     *
     * @return   object         Server response containing search results
     */
    function search_by_module($searchString,$modules,$max=1,$offset=0) {

      $sModules = is_array($modules)?$modules:explode(',',$modules);
      $response = $this->soapClient->search_by_module($this->userID,$this->userPWHash,$searchString,$sModules,$offset,$max);

      return $response;      

    }

    /*********************
     * Writing functions *
     *********************/

    /**
     * Writes a record to SugarCRM
     *
     * @param    string    $module: Module to store record in
     * @param    array     $newValues: Values of record fields
     *
     * @return   boolean   true if successful, false otherwise
     */
    function set_entry($module,$newValues) {

      $nameValues = $this->create_nameValueList($newValues);

      // have to use non-WSDL mode
      $response = $this->nonWsdlSoapClient->set_entry($this->nonWsdlSessionID,$module,$nameValues);

      return $response;

    }

    /********************
     * Helper functions *
     ********************/

    /**
     * Takes associative array and transforms it into a SugarCRM name_value_list
     *
     * @return   array   Key/value pairs as a name_value_list
     */
    function create_NameValueList($values) {
      
      reset($values); // just to make sure

      $i = 0;
      while(list($key, $val) = each($values)) {
	$nameValueList[$i]['name'] = $key;
	$nameValueList[$i]['value'] = $val;
	$i++;
      }
      return $nameValueList;

    }

    /**
     * Takes a SugarCRM name_value_list and converts it into an associative array
     *
     * @return   array   Associative array of name/value pairs from name_value_list
     */
    function convert_NameValueList($nameValueList) {
      
      reset($nameValueList); // just to make sure

      // array to save values in
      $assocRecord = array();

      foreach ($nameValueList as $nameValue) {
	$assocRecord[$nameValue->name] = $nameValue->value;
      }
      return $assocRecord;

    }

}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/eu_sugarcrm/model/class.tx_eusugarcrm_SOAP.php'])    {
    require_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/eu_sugarcrm/model/class.tx_eusugarcrm_SOAP.php']);
 }

?>
