<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Christian Lerrahn <christian.lerrahn@cerebrum.com.au>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 * Hint: use extdeveval to insert/update function index above.
 */

/**
 * Local API to SugarCRM for the 'eu_sugarcrm' extension.
 *
 * @author    Christian Lerrahn <christian.lerrahn@cerebrum.com.au>
 * @package    TYPO3
 * @subpackage    tx_eusugarcrm
 */

// load SugarCRM SOAP API
require_once('class.tx_eusugarcrm_SOAP.php');

class tx_eusugarcrm_SugarAPI {
  public $conf;
  protected $serviceObj;
  protected $userInput;

  function __construct() {
    
    // create the SOAP backend object
    $this->serviceObj = new tx_eusugarcrm_SOAP();

    // read default configuration
    $this->conf = $GLOBALS['TSFE']->config['config']['tx_eusugarcrm.'];
  }

  /**
   * Writes data to SugarCRM
   * @param    array     $userInput: Variables as passed to eu_sugarcrm
   * @return   boolean   True if SOAP object could be created, false if object creation failed.
   */
  function writeToSugar($userInput,$type=false) {

    $this->userInput = $userInput;

    if (is_object($this->serviceObj)) {

      // if we're submitting from a form we need to read config from DB
      if (!$type) {
	$this->conf = $this->getConfigFromDB();
      }

      // login to SugarCRM
      // WSDL (read)
      $this->serviceObj->login(true);
      // non-WSDL (write)
      $this->serviceObj->login(false);

      // iterate over objects to create
      while(list(, $obj) = each($this->conf)) {

	$this->createObject($obj);
      }

      // logout from SugarCRM
      // WSDL (write)
      $this->serviceObj->logout(true);      
      // non-WSDL (read)
      $this->serviceObj->logout(false);      

      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Read data from SugarCRM
   * @param   array    $data: Data to override with SugarCRM record
   * @param   string   $table: Database table to store data in after synchronisation
   *
   * @return  boolean  true on success, false otherwise
   */
  function readFromSugar($data,$table) {

    if (is_object($this->serviceObj)) {

      // login
      $this->serviceObj->login();

      // iterate over objects to read
      while(list(, $obj) = each($this->conf)) {

	if ($obj['identifier']) { // only if unique identifier set
	  $matchField = $data[$obj['mapping.'][$obj['identifier']]]?$data[$obj['mapping.'][$obj['identifier']]]:$data[$obj['identifier']];
	  if (!empty($matchField)) { // only search string not empty!
	    
	    $sugarData = $this->readObject($obj,$matchField);
	    
	    // map SugarCRM response onto array as laid out in DB
	    foreach ($sugarData->entry_list[0]->name_value_list as $idx => $field) {
	      if ($dbFieldName = $obj['mapping.'][$field->name]) {
		$data[$dbFieldName] = $field->value;
	      }
	    }

	    // write record to DB
	    $this->writeToDB($data,$table);
	  }

	}
      }

      // logout
      $this->serviceObj->logout();      

      return true;
    }
    else {
      return false;
    }

  }
  
  /********************
   * Helper functions *
   ********************/

  /**
   * Searches matching object in SugarCRM and updates local record
   *
   * @param    array    $config: TS configuration for SugarCRM synchronisation
   * @param    string   $sString: Search string
   * 
   * @return   array    Record from SugarCRM
   */
  function readObject($config,$sString) {

    // retrieve unique identifier
    $searchField = $config['identifier']?$config['identifier']:'';

    // set time shift from configuration (used for time stamps)
    $this->tshift = $config['timeshift'];

    // read record type in SugarCRM
    $sugarTable = $config['type'];

    // if not case sensitive, convert search string to lower case
    $sString = $config['identifier.']['case']?$sString:strtolower($sString);
      
    // find all records that have this identifier in SugarCRM
    $allMatches = $this->serviceObj->get_entry_list($sugarTable,$searchField.'="'.$sString.'"');
    //$allMatches = $this->serviceObj->search_by_module($sString,array($sugarTable));

    // find unique match (if any)
    $recID = $this->findUniqueMatch($allMatches,$searchField,$sString,$sugarTable);
    
    foreach ($config['mapping.'] as $key => $value) {
	       $selectFields .= $key.',';
    }
    $readRec = $this->serviceObj->get_entry($sugarTable,$recID,$selectFields);

    return $readRec;
  }

  /**
   * Creates a record on SugarCRM according to TS configuration using input data
   *
   * @param    array    $obj: TS configuration for SugarCRM synchronisation
   * @param    string   $parent: Parent object (only needed in recursion)
   * 
   * @return   integer  ID in SugarCRM
   */
  function createObject($obj, $parent=null) {

    // check if object is active
    if (!$obj['condition.'] || ($obj['condition.'] && $this->checkCondition($obj['condition.']))) {

      // read out time shift for this object
      $this->tshift = $obj['timeshift'];

      // read out the SugarCRM table we refer to
      $sugarTable = $obj['type'];

      // map input values to SugarCRM record
      $newRecord = $this->mergeValues($this->userInput,$obj['default.'], $obj['mapping.'], $dateformat, $parent);

      // copy unique identifier label
      $uniqueIdent = $obj['identifier'];

      // if any field set to unique, then overwrite
      if ($uniqueIdent) {
	// get the local value of the unique field
	// if not case sensitive, convert to lower case
	$uniqueVal = $obj['identifier.']['case']?$newRecord[$uniqueIdent]:strtolower($newRecord[$uniqueIdent]);

      }
      else { // default to "email1"
	$uniqueIdent = FALSE;
	$uniqueVal = '';
      }
      if ($uniqueIdent) {
	// find all records that have this identifier in SugarCRM
	$allMatches = $this->serviceObj->get_entry_list($sugarTable,$uniqueIdent.'="'.$uniqueVal.'"');
	//$allMatches = $this->serviceObj->search_by_module($uniqueVal,array($sugarTable));
	
	// find unique match (if any)
	$objID = $this->findUniqueMatch($allMatches,$uniqueIdent,$uniqueVal,$sugarTable);
      }
      else {
	$objID = FALSE;
      }

      // set ID of record to write to SugarCRM with matching ID
      if ($objID) {
	$newRecord['id'] = $objID;
      }
      
      // write data to SugarCRM
      $sugarRecord = $this->serviceObj->set_entry($sugarTable,$newRecord);

      $recID = $sugarRecord->id;
      $sugarRecord->module_name = $sugarTable;
      if (is_array($obj['children.'])) {
	foreach ($obj['children.'] as $child) {
	  $this->createObject($child, $sugarRecord);
	}
      }
      return $objID;
    }
  }

  /**
   * Writes a record to a database table
   *
   * @param     array    record data
   * @param     string   table to save to
   *
\   * @return    void
   */
  function writeToDB($data,$table) {

    $res = $GLOBALS['TYPO3_DB']->exec_UPDATEquery($table,'uid='.$data['uid'],$data);

  }

  
  /**
   * Finds unique match in a group of possible matches from free text search
   *
   * @param        string    $searchField: Name of field to match
   * @param        string    $sString: Search string
   *
   * @return       string    ID of (last) matching record
   */
  function findUniqueMatch($allMatches,$searchField,$sString,$table='') {

    // reset ID of found record
    $recId = false;
    foreach ($allMatches->entry_list as $key => $retObj) {

      // convert name_value_list to associative array
      $sugarRecord = $this->serviceObj->convert_NameValueList($retObj->name_value_list);

      // search field matches string?
      if ($sugarRecord[$searchField] == $sString) {

	// if match is not unique, log a warning
	if ($recId) {
	  t3lib_div::devLog('Unique identifier is not unique in in SugarCRM!','eu_sugarcrm',2);
	}

	// set record ID to current record
	$recId = $sugarRecord['id'];

      }
    }

    // no match? try querying all possible matches for full records
    if (($recId === false) && $table) {
      foreach ($allMatches->entry_list as $key => $retObj) {
	
	// convert name_value_list to associative array
	$sugarRecord = $this->serviceObj->convert_NameValueList($retObj->name_value_list);

	foreach ($this->conf['mapping.'] as $key => $value) {
	  $selectFields .= $key.',';
	}

	$readRec = $this->serviceObj->get_entry($table,$sugarRecord['id'],$selectFields);

	$recId = $this->findUniqueMatch($readRec,$searchField,$sString);
      }
    }

    return $recId;

  }
  
  /**
   * Checks if conditions for existence of object are met
   * 
   * @param     string   $condition: condition of existence for object
   *
   * @return	boolean  True if object exists, false otherwise
   */	
  function checkCondition($condition) {
    
    $create=1;
    $and = ($condition['or'])?0:1;
    if ($condition['form']) {
      $create = $this->evalLogicalExpression($condition['form']);
    }
    if (!$create)
      return false;
    
  }

  /**
   * Evaluates a simple logical expression
   * 
   * @param     string   $exp: logical expression to evaluate
   *
   * @return	boolean  Logical value of evaluated expression
   */	
  function evalLogicalExpression($exp) {
    
    // determine comparison operator first
    $pos = strpos($exp,'=');
    $posn = strpos($exp,'!');
    if (($pos !== FALSE) && ($posn == ($pos-1))) {
      $pos = $posn;
      $op = '!=';
    }
    elseif ($pos !== FALSE) {
      $op = '=';
    }
    $posn = strpos($exp,'<');
    if (($pos === FALSE) || (($posn === TRUE) && ($posn < $pos))) {
      $pos = $posn;
      $op = '<';
      $posn = strpos($exp,'<=');
      if ($posn = $pos) {
	$pos = $posn;
	$op = '<=';
      }
    }
    $posn = strpos($exp,'>');
    if (($pos === FALSE) || (($posn === TRUE) && ($posn < $pos))) {
      $pos = $posn;
      $op = '>';
      $posn = strpos($exp,'>=');
      if ($posn = $pos) {
	$pos = $posn;
	$op = '>=';
      }
    }

    // get variable name and value
    if ($op)
      list($cVar,$cVal) = explode($op,$exp,2);

    // evaluate expression
    switch ($op) {
    case '=':
      return ((trim($this->userInput[$cVar]) == trim($cVal)) || (trim($this->userInput[$cVar]) == intval($cVal)));
      break;      
    case '!=':
      return ((trim($this->userInput[$cVar]) != trim($cVal)) || (trim($this->userInput[$cVar]) != intval($cVal)));
      break;
    case '>':
      return ($this->userInput[$cVar] > intval($cVal));
      break;
    case '<':
      return ($this->userInput[$cVar] < intval($cVal));
      break;
    case '>=':
      return ($this->userInput[$cVar] >= intval($cVal));
      break;
    case '<=':
      return ($this->userInput[$cVar] <= intval($cVal));
      break;
    default:
      return 0;
    }

  }


  /**
   * Maps local values to their respective fields in SugarCRM
   * 
   * @param	array	 $recordData: record worked on
   * @param     array    $default: default mapping for top level object
   * @param     array    $mapping: local mapping for current object
   * @param     string   $dateformat: format for time stamps (default: m/d/Y)
   * @param     array    $parent: parent object's configuration
   *
   * @return	array 	 multidimensional TS array
   */	
  function mergeValues($recordData,$default,$mapping,$dateformat='m/d/Y',$parent=null) {

    // map values to correct SugarCRM field names
    while(list($key, $val) = each($mapping)) {
      $mapped[$key] = $recordData[$val];
    }

    // merge local mapping with object default
    if (is_array($default))
      $mapped = array_merge($default,$mapped);

    // merge parent values, if needed
    foreach ($mapped as $key => $val) {
      $valueParts = explode('.',$val);
      if (is_array($parent) && strtolower($valueParts[0]) == 'parent') {
	$mapped[$key] = $parent[strtolower($valueParts[1])];
	if ($valueParts[1] == 'type') $mapped[$key] = $parent['module_name'];
      }

      // find date fields and write timestamps in right format
      if (substr($key,0,5) == 'date_') {
	if ($date = $this->validateTimeStamp($mapped[$key], $dateformat)) { // time stamp has correct format
	  if (strtotime($date) > time()) { // time is in the future

	    // correct timestamp by configured time shift and format
	    $date = strftime('%Y-%m-%d %H:%M',(strtotime($date)-3600*$this->tshift));

	    // save into record
	    $mapped[$key] = $date;

	  }
	  else { // time is not in the future

	    // set field to default
	    $this->setDefaultDate($mapped,$default,$key,'%Y-%m-%d');

	  }
	}
	else { // format and time step do not match

	  // set field to default
	  $this->setDefaultDate($mapped,$default,$key,'%Y-%m-%d');

	}
      }
    }

    return $mapped;

  }


  /**
   * Sets a time stamp to a default.
   * Directly modifies the new record.
   *
   * @param  array   $record: new record passed as reference
   * @param  array   $default: "default" section of object configuration
   * @param  array   $key: name of date field
   * @param  string  $dateformat: format string for time stamps (default: '%Y-%m-%d)
   * 
   */	
  function setDefaultDate(&$record,$default,$key,$dateformat='%Y-%m-%d') {

    if ($default[$key]) {
      if ($default[$key] == 'now') {
	$record[$key] = gmstrftime($dateformat,time());
      } else {
	$record[$key] = $default[$key];
      }
    } else {
      unset($record[$key]);
    }

  }


  /**
   * Validates an input time stamp against a pre-defined format
   *
   * @param  string   $inputdate: time stamp as formatted string
   * @param  string   $dateformat: Foramt string to compare time stamp format to
   * 
   * @return  boolean  True if time stamp and format match, false otherwise
   */	
  function validateTimeStamp($inputdate,$dateformat) {
    
    // build regular expression
    $format = str_replace('d', '([0-9]{1,2})', $dateformat);
    $format = str_replace('m', '([0-9]{1,2})', $format);
    $format = str_replace('y', '([0-9]{2})', $format);
    $format = str_replace('Y', '([0-9]{4})', $format);
    $format = str_replace('H', '([0-1][0-9])', $format);
    $format = str_replace('M', '([0-6][0-9])', $format);
    $format = str_replace('.', '\.', $format);
    $format = '^'.$format.'$';
		
    $dateorder = ereg_replace('[^dmyYHM]', '', $dateformat);
    if (ereg($format, $inputdate, $matches)) {
      for ($jj=1; $jj<count($matches); $jj++) {
	switch (substr($dateorder, $jj-1, 1)) {
	case 'd':
	  $day = $matches[$jj];
	  break;
	case 'm':
	  $month = $matches[$jj];
	  break;
	case 'Y';
	case 'y':
	  $year = $matches[$jj];
	  break;
	case 'H':
	  $hour = $matches[$jj];
	  break;
	case 'M':
	  $minute = $matches[$jj];
	  break;
	}
      }
      if (checkdate($month, $day, $year)) {
	return $year.'-'.$month.'-'.$day.' '.sprintf('%02d',$hour).':'.sprintf('%02d',$minute).':00';
      } else {
	return false;
      }
    } else {
      return false;
    }

  }

  /**
   * Reads configuration from DB in the case of a mail form
   * 
   * @return  array  Parsed TS configuration  
   */
  function getConfigFromDB() {

    $post = t3lib_div::_POST();
    $locationData = $post['locationData'];
    $locData = explode(':',$locationData);
    $elUid = $locData[2];
    $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
						   'tx_eusugarcrm_mapping, tx_eusugarcrm_createlead',
						   'tt_content',
						   'uid = '.$elUid
						   );
    // if record was found, fetch contents
    if ($res)
      $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);

    // only parse configuration if SugarCRM synch enabled
    if ($row['tx_eusugarcrm_createlead']) {
      // load mapping
      $configTS = $row['tx_eusugarcrm_mapping'];
      $TSFromDB = $this->getConf($configTS);
    }

    return $TSFromDB;
  }

  /**
   * Parses TS configuration in a string into an array
   * 
   * @param	string	$config: TS configuration
   * @return	array 	multidimensional TS array
   */
  function getConf($config)	{
    // parsing is done by an instance of the t3lib_TSparser
    if(!class_exists('t3lib_TSparser') && defined('PATH_t3lib')) {
      require_once(PATH_t3lib.'class.t3lib_TSparser.php');
    }
    $parser = t3lib_div::makeInstance('t3lib_TSparser');
    $parser->parse($config);

    // after the TS is parsed we delete all configurations which do not fit the type AND
    // which are not enabled
    reset($parser->setup);
		
    $conf = $parser->setup;
    return $conf;
  }

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/eu_sugarcrm/model/class.tx_eusugarcrm_SugarAPI.php'])    {
    require_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/eu_sugarcrm/model/class.tx_eusugarcrm_SugarAPI.php']);
 }

?>
