<?php
  /***************************************************************
   *  Copyright notice
   *
   *  (c) 2005 Norman Seibert (seibert@entios.de)
   *  All rights reserved
   *
   *  This script is part of the TYPO3 project. The TYPO3 project is
   *  free software; you can redistribute it and/or modify
   *  it under the terms of the GNU General Public License as published by
   *  the Free Software Foundation; either version 2 of the License, or
   *  (at your option) any later version.
   *
   *  The GNU General Public License can be found at
   *  http://www.gnu.org/copyleft/gpl.html.
   *
   *  This script is distributed in the hope that it will be useful,
   *  but WITHOUT ANY WARRANTY; without even the implied warranty of
   *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   *  GNU General Public License for more details.
   *
   *  This copyright notice MUST APPEAR in all copies of the script!
   ***************************************************************/
  /**
   * Plugin 'SugarCRM Test' for the 'eu_sugarcrm' extension.
   * Various tests for SOAP connection.
   *
   * @author	Christian Lerrahn (Cerebrum (Aust) Pty. Ltd.) <christian.lerrahn@cerebrum.com.au>
   * @author	Norman Seibert <seibert@entios.de>
   */


require_once(t3lib_extMgm::extPath('eu_sugarcrm').'/model/class.tx_eusugarcrm_SOAP.php');

class tx_eusugarcrm_pi1 extends tslib_pibase {
  var $prefixId = 'tx_eusugarcrm_pi1';		// Same as class name
  var $scriptRelPath = 'pi1/class.tx_eusugarcrm_pi1.php';	// Path to this script relative to the extension dir.
  var $extKey = 'eu_sugarcrm';	// The extension key.

  public $conf; // configuration array
  protected $serviceObject; // SOAP client object

  /**
   * Tests the SugarCRM SOAP connection
   * Allows for different tests that can be configured via FlexForms
   *
   * @param  string  $content: Current content from TYPO3 core
   * @param  array   $conf: Plug-in configuration array
   */
  function main($content,$conf)	{
    
    // copy configuratio into global variable
    $this->conf=$conf;

    // init flexforms to $this->cObj->data['pi_flexform']
    $this->pi_initPIflexForm();

    // Configuring so caching is not expected. This value means that no cHash params are ever set. We do this, because it's a USER_INT object!
    $this->pi_USER_INT_obj=1;

    // Parameter for ping test
    $testparam = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'testparam', 'sDEF');

    // Test type to run
    $what_to_display = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'what_to_display', 'sDEF');
		
    // create SOAP object
    if (is_object($this->serviceObj = new tx_eusugarcrm_SOAP())) {

      // ping test if string to ping or no test set
      if ($testparam || !$what_to_display)
	$content .= '<b>'.$this->serviceObj->test($testparam).'</b>';

      // query server version and SugarCRM flavour and display it
      if (t3lib_div::inList($what_to_display,'SERVER_VERSION'))
	$content .= '<p>The server version is '.$this->serviceObj->get_server_version().' ('.$this->serviceObj->get_sugar_flavor().').</p>';

      // more sophisticated tests that require login
      if (t3lib_div::inList($what_to_display,'LOG_IN') || t3lib_div::inList($what_to_display,'LIST_USERS')
	  || t3lib_div::inList($what_to_display,'LIST_MODULES') || t3lib_div::inList($what_to_display,'LIST_USERS')
	  || t3lib_div::inList($what_to_display,'LIST_MODULE_DETAILS')) {

	// login to SugarCRM via SOAP
	if ($this->serviceObj->login()) {
	  $content .= '<p>Logged in to SugarCRM successfully!</p>';

	  // do we want to list all available users?
	  if (t3lib_div::inList($what_to_display,'LIST_USERS')) {
	    $allUsers = $this->serviceObj->get_user_list();
	    $content .= '<p>SugarCRM users:</p>';
	    $content .= '<table border=1>';
	    foreach ($allUsers as $user) { // write out all users, one by one
	      $content .= '<tr><td colspan=2><b>'.$user->user_name.'</b></td></tr>';
	   
	      // write out all user properties
	      foreach ($user as $key => $val) {
		$content .= '<tr><td>'.$key.':</td><td>'.$val.'</td></tr>';
	      }
	      $content .= '</tr>';
	    }
	    $content .= '</table>';
	  }
	  
	  // list all available SugarCRM modules
	  if (t3lib_div::inList($what_to_display,'LIST_MODULES') || t3lib_div::inList($what_to_display,'LIST_MODULE_DETAILS')) {
	    $allModules = $this->serviceObj->get_module_list();
	    $content .= '<p>SugarCRM modules:</p>';
	    $content .= '<table border=1>';

	    // iterate over all available modules
	    foreach ($allModules->modules as $mod) {

	      // write out module name
	      $content .= '<tr><td colspan=2><b>'.$mod.'</b></td></tr>';

	      // want to display module details, too?
	      if (t3lib_div::inList($what_to_display,'LIST_MODULE_DETAILS')) { 

		// retrieve lists of fields in module
		try {
		  $allFields = $this->serviceObj->get_module_fields($mod);
		}
		catch (SoapFault $soapException) {
		  // this is probably a SugarCRM bug
		  // we just ignore it! :)
		  unset($allFields);
		}

		// any fields?
		if ($allFields) {
		  $content .= '<tr><td><strong>name</strong></td>
								<td><strong>type</strong></td>
								<td><strong>required?</strong></td>
								<td><strong>options</strong></td>
							</tr>';

		  foreach ($allFields->module_fields as $field) {

		    // write out field details
		    $content .= '<tr><td>'.$field->name.':</td>
								<td>'.$field->type.'</td>
								<td>'.$field->required.'</td>
								<td>';
		    foreach ($field->options as $opt) {
		      
		      // write out options
		      $content .= '<div>'.$opt->name.': '.$opt->value.'</div>';
		    }
		    $content .= '</td>
							</tr>';
		  }
		}
	      }
	    }
	    $content .= '</table>';
	  }
	}
	else { // login failed
	  $content .= '<p>Failed to log in to SugarCRM!</p>';
	}
      }
      $this->serviceObj->logout();
    }
    return $this->pi_wrapInBaseClass($content);
  }
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/eu_sugarcrm/pi1/class.tx_eusugarcrm_pi1.php'])	{
  require_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/eu_sugarcrm/pi1/class.tx_eusugarcrm_pi1.php']);
 }

?>
